
#include "ar_cubes.h"

int main(){

    Sandbox sandbox;
    char *sandbox_conf_file = (char*)"../../sandbox_conf.yaml";
  
    if(sandbox.loadConfigFrom(sandbox_conf_file)){
        std::cout << "Failed to load the configuration" << std::endl;
        return 1;
    }
    
    if(sandbox.init()){
        std::cout << "Failed to initilize sandbox" << std::endl;
        return 1;
    }

    arCubes(sandbox);
}

void arCubes(Sandbox &sandbox){

    char wn[] = "Cubes";
    cv::namedWindow(wn, CV_WINDOW_NORMAL);
    cv::setWindowProperty(wn, CV_WND_PROP_FULLSCREEN, CV_WINDOW_FULLSCREEN);

    cv::Mat gray;

    
    cv::Mat blue;
    sandbox.captureFrame();
    cv::Mat src = sandbox.getColorFrame();
    cv::Mat dst = cv::Mat::zeros(src.rows, src.cols,  CV_8UC3);
    cv::Mat z = cv::Mat::zeros(src.rows, src.cols, CV_8UC1);
    
    std::vector<std::vector<cv::Point>> contours;
    std::vector<cv::Mat1b> channels;


    do{
        sandbox.captureFrame();
        src = sandbox.getColorFrame();
        dst = cv::Mat::zeros(src.rows, src.cols, CV_8UC3);
        contours.clear();
        channels.clear();

        cv::extractChannel(src, blue, 2);
        cv::threshold(blue, blue, 240, 255, cv::THRESH_TOZERO);
        
        channels.push_back(z);
        channels.push_back(z);
        channels.push_back(blue);

        cv::merge(channels, dst);

        //cv::cvtColor(src, gray, CV_RGB2GRAY, 0);
        //cv::threshold(gray, gray, 127,255,0);
        cv::findContours( blue, contours,  cv::RETR_EXTERNAL, cv::CHAIN_APPROX_SIMPLE);
        //cv::cvtColor(red, dst, CV_GRAY2RGB, 0);
        std::cout << "total " << contours.size() << std::endl;
        for (int i = 0; i < static_cast<int>(contours.size()); ++i) {
            //std::vector<cv::Point> approx = contours.at(i);
            std::vector<cv::Point> approx;
            double epsilon = 0.1*cv::arcLength(contours.at(i),true);
            cv::approxPolyDP(contours.at(i), approx, epsilon, true);

            std::cout << "count " << approx.size() << std::endl;
            cv::drawContours(dst, (std::vector<std::vector<cv::Point>>){approx}, 0, cv::Scalar(0,255,0));
        }
        cv::cvtColor(dst, dst, CV_RGB2BGR);
        cv::imshow(wn, dst);

    } while (cv::waitKey(10) != 27);

    
    cv::destroyAllWindows();

    // https://docs.opencv.org/trunk/dd/d49/tutorial_py_contour_features.html

    /*
    for cnt in contours:
            approx = cv2.approxPolyDP(cnt, 0.01*cv2.arcLength(cnt, True), True)
            cv2.drawContours(img, [approx], 0, (0), 5)
            x = approx.ravel()[0]
            y = approx.ravel()[1]
            if len(approx) == 3:
                cv2.putText(img, "Triangle", (x, y), font, 1, (0))
            elif len(approx) == 4:
                cv2.putText(img, "Rectangle", (x, y), font, 1, (0))
            elif len(approx) == 5:
                cv2.putText(img, "Pentagon", (x, y), font, 1, (0))
            elif 6 < len(approx) < 15:
                cv2.putText(img, "Ellipse", (x, y), font, 1, (0))
            else:
                cv2.putText(img, "Circle", (x, y), font, 1, (0))
    */
}
