
#include <sandbox.h>
#include <opencv2/opencv.hpp>

#include <chrono>
#include <iostream>
#include <fstream>

void testPerformances(Sandbox &sandbox, int cycle, bool take_pic);

int main(){

    Sandbox sandbox;
    char *sandbox_conf_file = (char*)"./sandbox_conf.yaml";

    if(sandbox.loadConfig(sandbox_conf_file)){
        std::cout << "Failed to load the configuration" << std::endl;
        return 1;
    }
    
    if(sandbox.init()){
        std::cout << "Failed to initilize sandbox" << std::endl;
        return 1;
    }

    // you can custom the number of cycle and if 
    // it has to take a picture at the beginning of the process
    testPerformances( sandbox, 200, false );
}



void testPerformances(Sandbox &sandbox, int cycle, bool take_pic){

    if(take_pic){
        sandbox.captureFrame();
        cv::Mat3b tmp_color = sandbox.getColorFrame();
        cv::Mat1f tmp_depth = sandbox.getDepthFrame();
        cv::imwrite("./frames/color.jpg", tmp_color);
        cv::FileStorage depth_file("./frames/depth.yml", cv::FileStorage::WRITE);
        depth_file << "depth" << tmp_depth;
        depth_file.release();
    }

    cv::Mat3b mockup_color = cv::imread("./frames/color.jpg");
    cv::Mat1f mockup_depth;
    cv::FileStorage depth_file("./frames/depth.yml", cv::FileStorage::READ);
    depth_file["depth"] >> mockup_depth;
    depth_file.release();

    if(mockup_color.empty() || mockup_depth.empty()){
        std::cout << "No mockup frames found" << std::endl;
        exit(0);
    }

    std::ofstream output;
    output.open ("./output/durations.txt");
    output << "capture_frame_ms,get_depth_frame_ms,get_color_frame_ms,adjust_projection_ms\n";
    
    // line : capture_frame_ms, get_depth_frame_ms, get_color_frame_ms, adjust_projection_ms
    for(int i = 0; i < cycle; i++){

        std::chrono::milliseconds start_capt_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        sandbox.captureFrame();
        std::chrono::milliseconds now_capt_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        int time_capt = (now_capt_ms - start_capt_ms).count();
        
        
        std::chrono::milliseconds start_depth_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        cv::Mat1f new_depth = sandbox.getDepthFrame();
        std::chrono::milliseconds now_depth_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        int time_depth = (now_depth_ms - start_depth_ms).count();


        std::chrono::milliseconds start_color_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        cv::Mat3b new_color = sandbox.getColorFrame();
        std::chrono::milliseconds now_color_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        int time_color = (now_color_ms - start_color_ms).count();
        

        std::chrono::milliseconds start_adj_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        cv::Mat3b result = sandbox.adjustProjection(mockup_color, new_depth);
        std::chrono::milliseconds now_adj_ms = std::chrono::duration_cast< std::chrono::milliseconds >( std::chrono::system_clock::now().time_since_epoch() );
        int time_adj = (now_adj_ms - start_adj_ms).count();

        output << time_capt << "," << time_depth << "," << time_color << "," << time_adj << std::endl;
    }

    output.close();
}
